@@ -25,8 +25,8 @@ pub use tag::Id3v2Tag;
 
 pub use items::*;
 
-pub use frame::id::FrameId;
-pub use frame::{Frame, FrameFlags, FrameValue};
+pub use frame::header::{FrameHeader, FrameId};
+pub use frame::{Frame, FrameFlags};
 
 pub use restrictions::{
 	ImageSizeRestrictions, TagRestrictions, TagSizeRestrictions, TextSizeRestrictions,
