@@ -2,7 +2,7 @@ use super::frame::read::ParsedFrame;
 use super::header::Id3v2Header;
 use super::tag::Id3v2Tag;
 use crate::config::ParseOptions;
-use crate::error::{Id3v2Error, Id3v2ErrorKind, Result};
+use crate::error::Result;
 use crate::id3::v2::util::synchsafe::UnsynchronizedStream;
 use crate::id3::v2::{Frame, FrameId, Id3v2Version, TimestampFrame};
 use crate::tag::items::Timestamp;
@@ -70,49 +70,49 @@ fn construct_tdrc_from_v3(tag: &mut Id3v2Tag) {
 		return;
 	}
 
-	let date = tag.get_text(&TDAT);
-	let mut date_used = false;
-
-	let time = tag.get_text(&TIME);
-	let mut time_used = false;
-
 	let mut tdrc = Timestamp {
 		year: year_frame.timestamp.year,
 		..Timestamp::default()
 	};
+
+	let mut date_used = false;
+	let mut time_used = false;
 	'build: {
-		if let Some(date) = date {
-			if date.len() != 4 {
-				log::warn!("Invalid TDAT frame, retaining.");
-				break 'build;
-			}
+		let Some(date) = tag.get_text(&TDAT) else {
+			break 'build;
+		};
 
-			let (Ok(day), Ok(month)) = (date[..2].parse::<u8>(), date[2..].parse::<u8>()) else {
-				log::warn!("Invalid TDAT frame, retaining.");
-				break 'build;
-			};
-
-			tdrc.month = Some(month);
-			tdrc.day = Some(day);
-			date_used = true;
-
-			if let Some(time) = time {
-				if time.len() != 4 {
-					log::warn!("Invalid TIME frame, retaining.");
-					break 'build;
-				}
-
-				let (Ok(hour), Ok(minute)) = (time[..2].parse::<u8>(), time[2..].parse::<u8>())
-				else {
-					log::warn!("Invalid TIME frame, retaining.");
-					break 'build;
-				};
-
-				tdrc.hour = Some(hour);
-				tdrc.minute = Some(minute);
-				time_used = true;
-			}
+		if date.len() != 4 || !date.is_ascii() {
+			log::warn!("Invalid TDAT frame, retaining.");
+			break 'build;
 		}
+
+		let (Ok(day), Ok(month)) = (date[..2].parse::<u8>(), date[2..].parse::<u8>()) else {
+			log::warn!("Invalid TDAT frame, retaining.");
+			break 'build;
+		};
+
+		tdrc.month = Some(month);
+		tdrc.day = Some(day);
+		date_used = true;
+
+		let Some(time) = tag.get_text(&TIME) else {
+			break 'build;
+		};
+
+		if time.len() != 4 || !time.is_ascii() {
+			log::warn!("Invalid TIME frame, retaining.");
+			break 'build;
+		}
+
+		let (Ok(hour), Ok(minute)) = (time[..2].parse::<u8>(), time[2..].parse::<u8>()) else {
+			log::warn!("Invalid TIME frame, retaining.");
+			break 'build;
+		};
+
+		tdrc.hour = Some(hour);
+		tdrc.minute = Some(minute);
+		time_used = true;
 	}
 
 	tag.insert(Frame::Timestamp(TimestampFrame::new(
@@ -130,19 +130,6 @@ fn construct_tdrc_from_v3(tag: &mut Id3v2Tag) {
 	}
 }
 
-fn skip_frame(reader: &mut impl Read, size: u32) -> Result<()> {
-	log::trace!("Skipping frame of size {}", size);
-
-	let size = u64::from(size);
-	let mut reader = reader.take(size);
-	let skipped = std::io::copy(&mut reader, &mut std::io::sink())?;
-	debug_assert!(skipped <= size);
-	if skipped != size {
-		return Err(Id3v2Error::new(Id3v2ErrorKind::BadFrameLength).into());
-	}
-	Ok(())
-}
-
 fn read_all_frames_into_tag<R>(
 	reader: &mut R,
 	header: Id3v2Header,
@@ -181,9 +168,7 @@ where
 				}
 			},
 			// No frame content found or ignored due to errors, but we can expect more frames
-			ParsedFrame::Skip { size } => {
-				skip_frame(reader, size)?;
-			},
+			ParsedFrame::Skip => {},
 			// No frame content found, and we can expect there are no more frames
 			ParsedFrame::Eof => break,
 		}
@@ -192,53 +177,59 @@ where
 	Ok(tag)
 }
 
-#[test]
-fn zero_size_id3v2() {
-	use crate::config::ParsingMode;
-	use crate::id3::v2::header::Id3v2Header;
+#[cfg(test)]
+mod tests {
+	use super::parse_id3v2;
+	use crate::config::ParseOptions;
 
-	use std::io::Cursor;
+	#[test_log::test]
+	fn zero_size_id3v2() {
+		use crate::config::ParsingMode;
+		use crate::id3::v2::header::Id3v2Header;
 
-	let mut f = Cursor::new(std::fs::read("tests/tags/assets/id3v2/zero.id3v2").unwrap());
-	let header = Id3v2Header::parse(&mut f).unwrap();
-	assert!(parse_id3v2(
-		&mut f,
-		header,
-		ParseOptions::new().parsing_mode(ParsingMode::Strict)
-	)
-	.is_ok());
-}
+		use std::io::Cursor;
 
-#[test]
-fn bad_frame_id_relaxed_id3v2() {
-	use crate::config::ParsingMode;
-	use crate::id3::v2::header::Id3v2Header;
-	use crate::prelude::*;
+		let mut f = Cursor::new(std::fs::read("tests/tags/assets/id3v2/zero.id3v2").unwrap());
+		let header = Id3v2Header::parse(&mut f).unwrap();
+		assert!(parse_id3v2(
+			&mut f,
+			header,
+			ParseOptions::new().parsing_mode(ParsingMode::Strict)
+		)
+		.is_ok());
+	}
 
-	use std::io::Cursor;
+	#[test_log::test]
+	fn bad_frame_id_relaxed_id3v2() {
+		use crate::config::ParsingMode;
+		use crate::id3::v2::header::Id3v2Header;
+		use crate::prelude::*;
 
-	// Contains a frame with a "+" in the ID, which is invalid.
-	// All other frames in the tag are valid, however.
-	let mut f = Cursor::new(
-		std::fs::read("tests/tags/assets/id3v2/bad_frame_otherwise_valid.id3v24").unwrap(),
-	);
-	let header = Id3v2Header::parse(&mut f).unwrap();
-	let id3v2 = parse_id3v2(
-		&mut f,
-		header,
-		ParseOptions::new().parsing_mode(ParsingMode::Relaxed),
-	);
-	assert!(id3v2.is_ok());
+		use std::io::Cursor;
 
-	let id3v2 = id3v2.unwrap();
+		// Contains a frame with a "+" in the ID, which is invalid.
+		// All other frames in the tag are valid, however.
+		let mut f = Cursor::new(
+			std::fs::read("tests/tags/assets/id3v2/bad_frame_otherwise_valid.id3v24").unwrap(),
+		);
+		let header = Id3v2Header::parse(&mut f).unwrap();
+		let id3v2 = parse_id3v2(
+			&mut f,
+			header,
+			ParseOptions::new().parsing_mode(ParsingMode::Relaxed),
+		);
+		assert!(id3v2.is_ok());
 
-	// There are 6 valid frames and 1 invalid frame
-	assert_eq!(id3v2.len(), 6);
+		let id3v2 = id3v2.unwrap();
 
-	assert_eq!(id3v2.title().as_deref(), Some("Foo title"));
-	assert_eq!(id3v2.artist().as_deref(), Some("Bar artist"));
-	assert_eq!(id3v2.comment().as_deref(), Some("Qux comment"));
-	assert_eq!(id3v2.year(), Some(1984));
-	assert_eq!(id3v2.track(), Some(1));
-	assert_eq!(id3v2.genre().as_deref(), Some("Classical"));
+		// There are 6 valid frames and 1 invalid frame
+		assert_eq!(id3v2.len(), 6);
+
+		assert_eq!(id3v2.title().as_deref(), Some("Foo title"));
+		assert_eq!(id3v2.artist().as_deref(), Some("Bar artist"));
+		assert_eq!(id3v2.comment().as_deref(), Some("Qux comment"));
+		assert_eq!(id3v2.year(), Some(1984));
+		assert_eq!(id3v2.track(), Some(1));
+		assert_eq!(id3v2.genre().as_deref(), Some("Classical"));
+	}
 }
