@@ -354,7 +354,7 @@ pub(crate) fn tagitems_into_riff<'a>(
 
 #[cfg(test)]
 mod tests {
-	use crate::config::WriteOptions;
+	use crate::config::{ParsingMode, WriteOptions};
 	use crate::iff::chunk::Chunks;
 	use crate::iff::wav::RiffInfoList;
 	use crate::prelude::*;
@@ -364,7 +364,7 @@ mod tests {
 
 	use std::io::Cursor;
 
-	#[test]
+	#[test_log::test]
 	fn parse_riff_info() {
 		let mut expected_tag = RiffInfoList::default();
 
@@ -383,13 +383,14 @@ mod tests {
 			&mut Chunks::<LittleEndian>::new(tag.len() as u64),
 			(tag.len() - 1) as u64,
 			&mut parsed_tag,
+			ParsingMode::Strict,
 		)
 		.unwrap();
 
 		assert_eq!(expected_tag, parsed_tag);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn riff_info_re_read() {
 		let tag = crate::tag::utils::test_utils::read_path("tests/tags/assets/test.riff");
 		let mut parsed_tag = RiffInfoList::default();
@@ -399,6 +400,7 @@ mod tests {
 			&mut Chunks::<LittleEndian>::new(tag.len() as u64),
 			(tag.len() - 1) as u64,
 			&mut parsed_tag,
+			ParsingMode::Strict,
 		)
 		.unwrap();
 
@@ -415,13 +417,14 @@ mod tests {
 			&mut Chunks::<LittleEndian>::new(tag.len() as u64),
 			(tag.len() - 13) as u64,
 			&mut temp_parsed_tag,
+			ParsingMode::Strict,
 		)
 		.unwrap();
 
 		assert_eq!(parsed_tag, temp_parsed_tag);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn riff_info_to_tag() {
 		let tag_bytes = crate::tag::utils::test_utils::read_path("tests/tags/assets/test.riff");
 
@@ -433,6 +436,7 @@ mod tests {
 			&mut Chunks::<LittleEndian>::new(tag_bytes.len() as u64),
 			(tag_bytes.len() - 1) as u64,
 			&mut riff_info,
+			ParsingMode::Strict,
 		)
 		.unwrap();
 
@@ -441,7 +445,7 @@ mod tests {
 		crate::tag::utils::test_utils::verify_tag(&tag, true, false);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn tag_to_riff_info() {
 		let tag = crate::tag::utils::test_utils::create_tag(TagType::RiffInfo);
 
