@@ -1,5 +1,7 @@
+pub(super) mod advisory_rating;
 pub(super) mod atom;
 pub(super) mod constants;
+pub(super) mod data_type;
 pub(super) mod read;
 mod r#ref;
 pub(crate) mod write;
@@ -15,7 +17,9 @@ use crate::tag::{
 };
 use crate::util::flag_item;
 use crate::util::io::{FileLike, Length, Truncate};
-use atom::{AdvisoryRating, Atom, AtomData};
+use advisory_rating::AdvisoryRating;
+use atom::{Atom, AtomData};
+use data_type::DataType;
 
 use std::borrow::Cow;
 use std::io::Write;
@@ -60,6 +64,32 @@ macro_rules! impl_accessor {
 	}
 }
 
+macro_rules! impl_flag_accessors {
+	($($name:ident ($ident:ident)),+ $(,)?) => {
+		$(
+			paste::paste! {
+				#[doc = "Whether the `" $ident "` flag atom is set"]
+				///
+				/// # Examples
+				///
+				/// ```rust
+				#[doc = "use lofty::mp4::constants::flags::" $name ";"]
+				/// use lofty::mp4::{AtomIdent, Ilst};
+				///
+				/// let mut ilst = Ilst::new();
+				///
+				/// // I want to toggle this flag!
+				#[doc = "ilst.set_flag(" $name ", true);"]
+				///
+				#[doc = "assert!(ilst.is_" $name:lower "());"]
+				pub fn [<is_ $name:lower>](&self) -> bool {
+					self.get_flag(&constants::flags::$name).unwrap_or(false)
+				}
+			}
+		)+
+	};
+}
+
 /// ## Pictures
 ///
 /// Unlike other formats, ilst does not store a [`PictureType`]. All pictures will have
@@ -345,6 +375,53 @@ impl Ilst {
 			.retain(|a| !matches!(a.data().next(), Some(AtomData::Picture(_))))
 	}
 
+	/// Sets the value of a flag ([`AtomData::Bool`]) atom
+	///
+	/// For identifiers, see [`constants::flags`].
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use lofty::mp4::constants::flags::COMPILATION;
+	/// use lofty::mp4::{AtomIdent, Ilst};
+	///
+	/// // This file part of a compilation!
+	/// let mut ilst = Ilst::new();
+	/// ilst.set_flag(COMPILATION, true);
+	///
+	/// assert!(ilst.is_compilation());
+	/// ```
+	pub fn set_flag(&mut self, ident: AtomIdent<'_>, value: bool) {
+		if !value {
+			// A flag with a value of `false` is equivalent to removing it.
+			let _ = self.remove(&ident);
+			return;
+		}
+
+		let data = AtomData::Bool(value);
+		self.replace_atom(Atom {
+			ident,
+			data: AtomDataStorage::Single(data),
+		});
+	}
+
+	fn get_flag(&self, ident: &AtomIdent<'_>) -> Option<bool> {
+		self.get(ident)
+			.and_then(|atom| atom.data().next())
+			.and_then(|data| match data {
+				AtomData::Bool(b) => Some(*b),
+				_ => None,
+			})
+	}
+
+	impl_flag_accessors!(
+		PODCAST(pcst),
+		GAPLESS(pgap),
+		SHOW_WORK(shwm),
+		HD_VIDEO(hdvd),
+		COMPILATION(cpil)
+	);
+
 	/// Returns the parental advisory rating according to the `rtng` atom
 	pub fn advisory_rating(&self) -> Option<AdvisoryRating> {
 		self.get(&ADVISORY_RATING)
@@ -372,7 +449,10 @@ impl Ilst {
 	fn extract_number(&self, fourcc: [u8; 4], expected_size: usize) -> Option<u16> {
 		if let Some(atom) = self.get(&AtomIdent::Fourcc(fourcc)) {
 			match atom.data().next() {
-				Some(AtomData::Unknown { code: 0, data }) if data.len() >= expected_size => {
+				Some(AtomData::Unknown {
+					code: DataType::Reserved,
+					data,
+				}) if data.len() >= expected_size => {
 					return Some(u16::from_be_bytes([
 						data[expected_size - 2],
 						data[expected_size - 1],
@@ -601,7 +681,10 @@ impl SplitTag for Ilst {
 					ItemValue::Text(text)
 				},
 				// We have to special case track/disc numbers since they are stored together
-				AtomData::Unknown { code: 0, data } if Vec::len(data) >= 6 => {
+				AtomData::Unknown {
+					code: DataType::Reserved,
+					data,
+				} if Vec::len(data) >= 6 => {
 					if let AtomIdent::Fourcc(ref fourcc) = ident {
 						match fourcc {
 							b"trkn" => {
@@ -684,7 +767,7 @@ impl MergeTag for SplitTagRemainder {
 					tag.atoms.push(Atom {
 						ident: AtomIdent::Fourcc(ident),
 						data: AtomDataStorage::Single(AtomData::Unknown {
-							code: 0,
+							code: DataType::Reserved,
 							data: vec![0, 0, current[0], current[1], total[0], total[1], 0, 0],
 						}),
 					})
@@ -758,7 +841,7 @@ impl MergeTag for SplitTagRemainder {
 			picture.pic_type = PictureType::Other;
 
 			merged.atoms.push(Atom {
-				ident: AtomIdent::Fourcc([b'c', b'o', b'v', b'r']),
+				ident: COVR,
 				data: AtomDataStorage::Single(AtomData::Picture(picture)),
 			})
 		}
@@ -800,31 +883,33 @@ mod tests {
 	use crate::mp4::ilst::atom::AtomDataStorage;
 	use crate::mp4::ilst::TITLE;
 	use crate::mp4::read::AtomReader;
-	use crate::mp4::{AdvisoryRating, Atom, AtomData, AtomIdent, Ilst, Mp4File};
+	use crate::mp4::{AdvisoryRating, Atom, AtomData, AtomIdent, DataType, Ilst, Mp4File};
+	use crate::picture::{MimeType, Picture, PictureType};
 	use crate::prelude::*;
 	use crate::tag::utils::test_utils;
 	use crate::tag::utils::test_utils::read_path;
 	use crate::tag::{ItemValue, Tag, TagItem, TagType};
 
-	use crate::picture::{MimeType, Picture, PictureType};
 	use std::io::{Cursor, Read as _, Seek as _, Write as _};
 
-	fn read_ilst(path: &str, parse_mode: ParsingMode) -> Ilst {
+	fn read_ilst(path: &str, parse_options: ParseOptions) -> Ilst {
 		let tag = std::fs::read(path).unwrap();
-		read_ilst_raw(&tag, parse_mode)
+		read_ilst_raw(&tag, parse_options)
 	}
 
-	fn read_ilst_raw(bytes: &[u8], parse_mode: ParsingMode) -> Ilst {
-		let options = ParseOptions::new().parsing_mode(parse_mode);
-		read_ilst_with_options(bytes, options)
+	fn read_ilst_raw(bytes: &[u8], parse_options: ParseOptions) -> Ilst {
+		read_ilst_with_options(bytes, parse_options)
 	}
 
 	fn read_ilst_strict(path: &str) -> Ilst {
-		read_ilst(path, ParsingMode::Strict)
+		read_ilst(path, ParseOptions::new().parsing_mode(ParsingMode::Strict))
 	}
 
 	fn read_ilst_bestattempt(path: &str) -> Ilst {
-		read_ilst(path, ParsingMode::BestAttempt)
+		read_ilst(
+			path,
+			ParseOptions::new().parsing_mode(ParsingMode::BestAttempt),
+		)
 	}
 
 	fn read_ilst_with_options(bytes: &[u8], parse_options: ParseOptions) -> Ilst {
@@ -841,7 +926,7 @@ mod tests {
 		assert_eq!(atom.data().next().unwrap(), data);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn parse_ilst() {
 		let mut expected_tag = Ilst::default();
 
@@ -851,7 +936,7 @@ mod tests {
 		expected_tag.insert(Atom::new(
 			AtomIdent::Fourcc(*b"trkn"),
 			AtomData::Unknown {
-				code: 0,
+				code: DataType::Reserved,
 				data: vec![0, 0, 0, 1, 0, 0, 0, 0],
 			},
 		));
@@ -860,7 +945,7 @@ mod tests {
 		expected_tag.insert(Atom::new(
 			AtomIdent::Fourcc(*b"disk"),
 			AtomData::Unknown {
-				code: 0,
+				code: DataType::Reserved,
 				data: vec![0, 0, 0, 1, 0, 2],
 			},
 		));
@@ -911,7 +996,7 @@ mod tests {
 		assert_eq!(expected_tag, parsed_tag);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn ilst_re_read() {
 		let parsed_tag = read_ilst_strict("tests/tags/assets/ilst/test.ilst");
 
@@ -934,7 +1019,7 @@ mod tests {
 		assert_eq!(parsed_tag, temp_parsed_tag);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn ilst_to_tag() {
 		let tag = crate::tag::utils::test_utils::read_path("tests/tags/assets/ilst/test.ilst");
 		let len = tag.len();
@@ -957,7 +1042,7 @@ mod tests {
 		assert_eq!(tag.get_string(&ItemKey::DiscTotal), Some("2"));
 	}
 
-	#[test]
+	#[test_log::test]
 	fn tag_to_ilst() {
 		let mut tag = crate::tag::utils::test_utils::create_tag(TagType::Mp4Ilst);
 
@@ -995,7 +1080,7 @@ mod tests {
 			&ilst,
 			*b"trkn",
 			&AtomData::Unknown {
-				code: 0,
+				code: DataType::Reserved,
 				data: vec![0, 0, 0, 1, 0, 0, 0, 0],
 			},
 		);
@@ -1003,13 +1088,13 @@ mod tests {
 			&ilst,
 			*b"disk",
 			&AtomData::Unknown {
-				code: 0,
+				code: DataType::Reserved,
 				data: vec![0, 0, 0, 1, 0, 2, 0, 0],
 			},
 		)
 	}
 
-	#[test]
+	#[test_log::test]
 	fn issue_34() {
 		let ilst = read_ilst_strict("tests/tags/assets/ilst/issue_34.ilst");
 
@@ -1022,13 +1107,13 @@ mod tests {
 			&ilst,
 			*b"plID",
 			&AtomData::Unknown {
-				code: 21,
+				code: DataType::BeSignedInteger,
 				data: 88888_u64.to_be_bytes().to_vec(),
 			},
 		)
 	}
 
-	#[test]
+	#[test_log::test]
 	fn advisory_rating() {
 		let ilst = read_ilst_strict("tests/tags/assets/ilst/advisory_rating.ilst");
 
@@ -1041,7 +1126,7 @@ mod tests {
 		assert_eq!(ilst.advisory_rating(), Some(AdvisoryRating::Explicit));
 	}
 
-	#[test]
+	#[test_log::test]
 	fn trailing_padding() {
 		const ILST_START: usize = 97;
 		const ILST_END: usize = 131;
@@ -1106,7 +1191,7 @@ mod tests {
 		assert!(Mp4File::read_from(&mut file, ParseOptions::new().read_properties(false)).is_ok());
 	}
 
-	#[test]
+	#[test_log::test]
 	fn read_non_full_meta_atom() {
 		let file_bytes = read_path("tests/files/assets/non_full_meta_atom.m4a");
 		let file = Mp4File::read_from(
@@ -1118,7 +1203,7 @@ mod tests {
 		assert!(file.ilst_tag.is_some());
 	}
 
-	#[test]
+	#[test_log::test]
 	fn write_non_full_meta_atom() {
 		// This is testing writing to a file with a non-full meta atom
 		// We will *not* write a non-full meta atom
@@ -1147,7 +1232,7 @@ mod tests {
 		);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn multi_value_atom() {
 		let ilst = read_ilst_strict("tests/tags/assets/ilst/multi_value_atom.ilst");
 		let artist_atom = ilst.get(&AtomIdent::Fourcc(*b"\xa9ART")).unwrap();
@@ -1168,7 +1253,7 @@ mod tests {
 		);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn multi_value_roundtrip() {
 		let mut tag = Tag::new(TagType::Mp4Ilst);
 		tag.insert_text(ItemKey::TrackArtist, "TrackArtist 1".to_owned());
@@ -1231,7 +1316,7 @@ mod tests {
 		assert_eq!(tag.items, split_tag.items);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn zero_sized_ilst() {
 		let file = Mp4File::read_from(
 			&mut Cursor::new(test_utils::read_path("tests/files/assets/zero/zero.ilst")),
@@ -1242,7 +1327,7 @@ mod tests {
 		assert_eq!(file.ilst(), Some(&Ilst::default()));
 	}
 
-	#[test]
+	#[test_log::test]
 	fn merge_insert() {
 		let mut ilst = Ilst::new();
 
@@ -1258,7 +1343,7 @@ mod tests {
 		assert_eq!(ilst.len(), 1);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn invalid_atom_type() {
 		let ilst = read_ilst_strict("tests/tags/assets/ilst/invalid_atom_type.ilst");
 
@@ -1272,7 +1357,7 @@ mod tests {
 		assert_eq!(ilst.disk_total().unwrap(), 2);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn invalid_string_encoding() {
 		let ilst = read_ilst_bestattempt("tests/tags/assets/ilst/invalid_string_encoding.ilst");
 
@@ -1287,7 +1372,7 @@ mod tests {
 		assert!(ilst.album().is_none());
 	}
 
-	#[test]
+	#[test_log::test]
 	fn flag_item_conversion() {
 		let mut tag = Tag::new(TagType::Mp4Ilst);
 		tag.insert_text(ItemKey::FlagCompilation, "1".to_owned());
@@ -1312,14 +1397,14 @@ mod tests {
 		);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn special_items_roundtrip() {
 		let mut tag = Ilst::new();
 
 		let atom = Atom::new(
 			AtomIdent::Fourcc(*b"SMTH"),
 			AtomData::Unknown {
-				code: 0,
+				code: DataType::Reserved,
 				data: b"Meaningless Data".to_vec(),
 			},
 		);
@@ -1344,7 +1429,10 @@ mod tests {
 
 		tag_bytes.drain(..8); // Remove the ilst identifier and size for `read_ilst`
 
-		let tag_re_read = read_ilst_raw(&tag_bytes[..], ParsingMode::Strict);
+		let tag_re_read = read_ilst_raw(
+			&tag_bytes[..],
+			ParseOptions::new().parsing_mode(ParsingMode::Strict),
+		);
 		assert_eq!(tag, tag_re_read);
 
 		// Now write from `Tag`
@@ -1356,11 +1444,14 @@ mod tests {
 
 		tag_bytes.drain(..8); // Remove the ilst identifier and size for `read_ilst`
 
-		let generic_tag_re_read = read_ilst_raw(&tag_bytes[..], ParsingMode::Strict);
+		let generic_tag_re_read = read_ilst_raw(
+			&tag_bytes[..],
+			ParseOptions::new().parsing_mode(ParsingMode::Strict),
+		);
 		assert_eq!(tag_re_read, generic_tag_re_read);
 	}
 
-	#[test]
+	#[test_log::test]
 	fn skip_reading_cover_art() {
 		let p = Picture::new_unchecked(
 			PictureType::CoverFront,
@@ -1382,4 +1473,49 @@ mod tests {
 		assert_eq!(ilst.len(), 1); // Artist, no picture
 		assert!(ilst.artist().is_some());
 	}
+
+	#[test_log::test]
+	fn gnre_conversion_case_1() {
+		// Case 1: 1 `gnre` atom present, no `©gen` present. `gnre` gets converted without issue.
+		let ilst = read_ilst_bestattempt("tests/tags/assets/ilst/gnre_conversion_case_1.ilst");
+
+		assert_eq!(ilst.len(), 2);
+		assert_eq!(ilst.artist().unwrap(), "Foo artist"); // Sanity check
+		assert_eq!(ilst.genre().unwrap(), "Funk");
+	}
+
+	#[test_log::test]
+	fn gnre_conversion_case_2() {
+		// Case 2: 1 `gnre` atom present, 1 `©gen` present. `gnre` gets discarded.
+		let ilst = read_ilst_bestattempt("tests/tags/assets/ilst/gnre_conversion_case_2.ilst");
+
+		assert_eq!(ilst.len(), 2);
+		assert_eq!(ilst.artist().unwrap(), "Foo artist"); // Sanity check
+		assert_eq!(ilst.genre().unwrap(), "My Custom Genre");
+	}
+
+	#[test_log::test]
+	fn gnre_conversion_case_3() {
+		// Case 2: 1 `gnre` atom present, 1 `©gen` present. implicit conversion are disabled, `gnre` is retained
+		//         as an unknown atom.
+		let ilst = read_ilst(
+			"tests/tags/assets/ilst/gnre_conversion_case_2.ilst",
+			ParseOptions::new().implicit_conversions(false),
+		);
+
+		assert_eq!(ilst.len(), 3);
+		assert_eq!(ilst.artist().unwrap(), "Foo artist"); // Sanity check
+		assert_eq!(ilst.genre().unwrap(), "My Custom Genre");
+		assert_eq!(
+			ilst.get(&AtomIdent::Fourcc(*b"gnre"))
+				.unwrap()
+				.data()
+				.next()
+				.unwrap(),
+			&AtomData::Unknown {
+				code: DataType::BeSignedInteger,
+				data: vec![0, 6]
+			}
+		);
+	}
 }
