@@ -1,15 +1,15 @@
-use super::constants::{
-	BE_SIGNED_INTEGER, BE_UNSIGNED_INTEGER, BMP, JPEG, PNG, RESERVED, UTF16, UTF8,
-};
+use super::constants::WELL_KNOWN_TYPE_SET;
+use super::data_type::DataType;
 use super::{Atom, AtomData, AtomIdent, Ilst};
 use crate::config::{ParseOptions, ParsingMode};
 use crate::error::{LoftyError, Result};
 use crate::id3::v1::constants::GENRES;
 use crate::macros::{err, try_vec};
-use crate::mp4::atom_info::AtomInfo;
+use crate::mp4::atom_info::{AtomInfo, ATOM_HEADER_LEN};
 use crate::mp4::ilst::atom::AtomDataStorage;
-use crate::mp4::read::{skip_unneeded, AtomReader};
+use crate::mp4::read::{skip_atom, AtomReader};
 use crate::picture::{MimeType, Picture, PictureType};
+use crate::tag::TagExt;
 use crate::util::text::{utf16_decode_bytes, utf8_decode};
 
 use std::borrow::Cow;
@@ -34,48 +34,62 @@ where
 
 	let mut tag = Ilst::default();
 
+	let mut upgraded_gnres = Vec::new();
 	while let Ok(Some(atom)) = ilst_reader.next() {
 		if let AtomIdent::Fourcc(ref fourcc) = atom.ident {
 			match fourcc {
 				b"free" | b"skip" => {
-					skip_unneeded(&mut ilst_reader, atom.extended, atom.len)?;
+					skip_atom(&mut ilst_reader, atom.extended, atom.len)?;
 					continue;
 				},
 				b"covr" => {
 					if parse_options.read_cover_art {
 						handle_covr(&mut ilst_reader, parsing_mode, &mut tag, &atom)?;
 					} else {
-						skip_unneeded(&mut ilst_reader, atom.extended, atom.len)?;
+						skip_atom(&mut ilst_reader, atom.extended, atom.len)?;
 					}
 
 					continue;
 				},
 				// Upgrade this to a \xa9gen atom
-				b"gnre" => {
+				b"gnre" if parse_options.implicit_conversions => {
 					log::warn!("Encountered outdated 'gnre' atom, attempting to upgrade to '©gen'");
 
+					if let Some(atom_data) =
+						parse_data_inner(&mut ilst_reader, parsing_mode, &atom)?
+					{
+						for (_, content) in atom_data {
+							if content.len() >= 2 {
+								let index = content[1] as usize;
+								if index > 0 && index <= GENRES.len() {
+									upgraded_gnres
+										.push(AtomData::UTF8(String::from(GENRES[index - 1])));
+								}
+							}
+						}
+					}
+
+					continue;
+				},
+				// Just insert these normally, the caller will deal with them (hopefully)
+				b"gnre" => {
+					log::warn!("Encountered outdated 'gnre' atom");
+
 					if let Some(atom_data) =
 						parse_data_inner(&mut ilst_reader, parsing_mode, &atom)?
 					{
 						let mut data = Vec::new();
 
-						for (_, content) in atom_data {
-							if content.len() >= 2 {
-								let index = content[1] as usize;
-								if index > 0 && index <= GENRES.len() {
-									data.push(AtomData::UTF8(String::from(GENRES[index - 1])));
-								}
-							}
+						for (code, content) in atom_data {
+							data.push(AtomData::Unknown {
+								code,
+								data: content,
+							});
 						}
 
-						if !data.is_empty() {
-							let storage = match data.len() {
-								1 => AtomDataStorage::Single(data.remove(0)),
-								_ => AtomDataStorage::Multiple(data),
-							};
-
+						if let Some(storage) = AtomDataStorage::from_vec(data) {
 							tag.atoms.push(Atom {
-								ident: AtomIdent::Fourcc(*b"\xa9gen"),
+								ident: AtomIdent::Fourcc(*b"gnre"),
 								data: storage,
 							})
 						}
@@ -120,10 +134,9 @@ where
 						parse_data_inner(&mut ilst_reader, parsing_mode, &atom)?
 					{
 						if let Some((_, content)) = atom_data.first() {
-							let data = match content[..] {
-								[0, ..] => AtomData::Bool(false),
-								_ => AtomData::Bool(true),
-							};
+							// Any size integer is technically valid, we'll correct it on write.
+							let is_true = content.iter().any(|&b| b != 0);
+							let data = AtomData::Bool(is_true);
 
 							tag.atoms.push(Atom {
 								ident: AtomIdent::Fourcc(*fourcc),
@@ -141,6 +154,20 @@ where
 		parse_data(&mut ilst_reader, parsing_mode, &mut tag, atom)?;
 	}
 
+	if parse_options.implicit_conversions && !upgraded_gnres.is_empty() {
+		if tag.contains(&AtomIdent::Fourcc(*b"\xa9gen")) {
+			log::warn!("Encountered '©gen' atom, discarding upgraded 'gnre' atom(s)");
+			return Ok(tag);
+		}
+
+		if let Some(storage) = AtomDataStorage::from_vec(upgraded_gnres) {
+			tag.atoms.push(Atom {
+				ident: AtomIdent::Fourcc(*b"\xa9gen"),
+				data: storage,
+			})
+		}
+	}
+
 	Ok(tag)
 }
 
@@ -205,7 +232,7 @@ fn parse_data_inner<R>(
 	reader: &mut AtomReader<R>,
 	parsing_mode: ParsingMode,
 	atom_info: &AtomInfo,
-) -> Result<Option<Vec<(u32, Vec<u8>)>>>
+) -> Result<Option<Vec<(DataType, Vec<u8>)>>>
 where
 	R: Read + Seek,
 {
@@ -238,40 +265,41 @@ where
 			break;
 		}
 
-		// We don't care about the version
-		let _version = reader.read_u8()?;
+		if next_atom.ident != DATA_ATOM_IDENT {
+			if parsing_mode == ParsingMode::Strict {
+				err!(BadAtom("Expected atom \"data\" to follow name"))
+			}
 
-		let mut flags = [0; 3];
-		reader.read_exact(&mut flags)?;
+			log::warn!(
+				"Skipping unexpected atom {actual_ident:?}, expected {expected_ident:?}",
+				actual_ident = next_atom.ident,
+				expected_ident = DATA_ATOM_IDENT
+			);
 
-		let flags = u32::from_be_bytes([0, flags[0], flags[1], flags[2]]);
+			pos += next_atom.len;
+			skip_atom(reader, next_atom.extended, next_atom.len)?;
+			continue;
+		}
+
+		let Some(data_type) = parse_type_indicator(reader, parsing_mode)? else {
+			log::warn!("Skipping atom with unknown type set");
+			let remaining_atom_len = next_atom.len - (ATOM_HEADER_LEN + 1);
+
+			reader.seek(SeekFrom::Current(remaining_atom_len as i64))?;
+			pos += remaining_atom_len;
+			continue;
+		};
 
 		// We don't care about the locale
 		reader.seek(SeekFrom::Current(4))?;
 
-		match next_atom.ident {
-			DATA_ATOM_IDENT => {
-				let content_len = (next_atom.len - 16) as usize;
-				if content_len > 0 {
-					let mut content = try_vec![0; content_len];
-					reader.read_exact(&mut content)?;
-					ret.push((flags, content));
-				} else {
-					log::warn!("Skipping empty \"data\" atom");
-				}
-			},
-			_ => match parsing_mode {
-				ParsingMode::Strict => {
-					err!(BadAtom("Expected atom \"data\" to follow name"))
-				},
-				ParsingMode::BestAttempt | ParsingMode::Relaxed => {
-					log::warn!(
-						"Skipping unexpected atom {actual_ident:?}, expected {expected_ident:?}",
-						actual_ident = next_atom.ident,
-						expected_ident = DATA_ATOM_IDENT
-					)
-				},
-			},
+		let content_len = (next_atom.len - 16) as usize;
+		if content_len > 0 {
+			let mut content = try_vec![0; content_len];
+			reader.read_exact(&mut content)?;
+			ret.push((data_type, content));
+		} else {
+			log::warn!("Skipping empty \"data\" atom");
 		}
 
 		pos += next_atom.len;
@@ -281,6 +309,29 @@ where
 	Ok(ret)
 }
 
+fn parse_type_indicator<R>(
+	reader: &mut AtomReader<R>,
+	parsing_mode: ParsingMode,
+) -> Result<Option<DataType>>
+where
+	R: Read + Seek,
+{
+	// The type indicator is formed of four bytes split between two fields. The first byte indicates
+	// the set of types from which the type is drawn. The second through fourth byte forms the second
+	// field and its interpretation depends upon the value in the first field.
+
+	let type_set = reader.read_u8()?;
+	if type_set != WELL_KNOWN_TYPE_SET {
+		if parsing_mode == ParsingMode::Strict {
+			err!(BadAtom("Unknown type set in data atom"))
+		}
+
+		return Ok(None);
+	}
+
+	Ok(Some(DataType::from(reader.read_u24()?)))
+}
+
 fn parse_uint(bytes: &[u8]) -> Result<u32> {
 	Ok(match bytes.len() {
 		1 => u32::from(bytes[0]),
@@ -318,16 +369,26 @@ where
 		let mut data = Vec::new();
 
 		let len = atom_data.len();
-		for (flags, value) in atom_data {
-			let mime_type = match flags {
+		for (data_type, value) in atom_data {
+			let mime_type = match data_type {
 				// Type 0 is implicit
-				RESERVED => None,
+				DataType::Reserved => None,
 				// GIF is deprecated
-				12 => Some(MimeType::Gif),
-				JPEG => Some(MimeType::Jpeg),
-				PNG => Some(MimeType::Png),
-				BMP => Some(MimeType::Bmp),
-				_ => err!(BadAtom("\"covr\" atom has an unknown type")),
+				DataType::Gif => Some(MimeType::Gif),
+				DataType::Jpeg => Some(MimeType::Jpeg),
+				DataType::Png => Some(MimeType::Png),
+				DataType::Bmp => Some(MimeType::Bmp),
+				_ => {
+					if parsing_mode == ParsingMode::Strict {
+						err!(BadAtom("\"covr\" atom has an unknown type"))
+					}
+
+					log::warn!(
+						"Encountered \"covr\" atom with an unknown type of `{}`, discarding",
+						Into::<u32>::into(data_type)
+					);
+					return Ok(());
+				},
 			};
 
 			let picture_data = AtomData::Picture(Picture {
@@ -358,13 +419,13 @@ where
 	Ok(())
 }
 
-fn interpret_atom_content(flags: u32, content: Vec<u8>) -> Result<AtomData> {
+fn interpret_atom_content(flags: DataType, content: Vec<u8>) -> Result<AtomData> {
 	// https://developer.apple.com/library/archive/documentation/QuickTime/QTFF/Metadata/Metadata.html#//apple_ref/doc/uid/TP40000939-CH1-SW35
 	Ok(match flags {
-		UTF8 => AtomData::UTF8(utf8_decode(content)?),
-		UTF16 => AtomData::UTF16(utf16_decode_bytes(&content, u16::from_be_bytes)?),
-		BE_SIGNED_INTEGER => AtomData::SignedInteger(parse_int(&content)?),
-		BE_UNSIGNED_INTEGER => AtomData::UnsignedInteger(parse_uint(&content)?),
+		DataType::Utf8 => AtomData::UTF8(utf8_decode(content)?),
+		DataType::Utf16 => AtomData::UTF16(utf16_decode_bytes(&content, u16::from_be_bytes)?),
+		DataType::BeSignedInteger => AtomData::SignedInteger(parse_int(&content)?),
+		DataType::BeUnsignedInteger => AtomData::UnsignedInteger(parse_uint(&content)?),
 		code => AtomData::Unknown {
 			code,
 			data: content,
