@@ -1,3 +1,4 @@
+use super::data_type::DataType;
 use super::r#ref::IlstRef;
 use crate::config::{ParseOptions, WriteOptions};
 use crate::error::{FileEncodingError, LoftyError, Result};
@@ -5,7 +6,7 @@ use crate::file::FileType;
 use crate::macros::{decode_err, err, try_vec};
 use crate::mp4::atom_info::{AtomIdent, AtomInfo, ATOM_HEADER_LEN, FOURCC_LEN};
 use crate::mp4::ilst::r#ref::AtomRef;
-use crate::mp4::read::{atom_tree, meta_is_full, nested_atom, verify_mp4, AtomReader};
+use crate::mp4::read::{atom_tree, find_child_atom, meta_is_full, verify_mp4, AtomReader};
 use crate::mp4::write::{AtomWriter, AtomWriterCompanion, ContextualAtom};
 use crate::mp4::AtomData;
 use crate::picture::{MimeType, Picture};
@@ -73,10 +74,10 @@ where
 	let ilst = build_ilst(&mut tag.atoms)?;
 	let remove_tag = ilst.is_empty();
 
-	let udta = nested_atom(
+	let udta = find_child_atom(
 		&mut write_handle,
 		moov_len,
-		b"udta",
+		*b"udta",
 		ParseOptions::DEFAULT_PARSING_MODE,
 	)?;
 
@@ -101,10 +102,10 @@ where
 		existing_udta_size = udta.len;
 		new_udta_size = existing_udta_size;
 
-		let meta = nested_atom(
+		let meta = find_child_atom(
 			&mut write_handle,
 			udta.len,
-			b"meta",
+			*b"meta",
 			ParseOptions::DEFAULT_PARSING_MODE,
 		)?;
 
@@ -675,12 +676,12 @@ where
 {
 	for value in data {
 		match value {
-			AtomData::UTF8(text) => write_data(1, text.as_bytes(), writer)?,
-			AtomData::UTF16(text) => write_data(2, text.as_bytes(), writer)?,
+			AtomData::UTF8(text) => write_data(DataType::Utf8, text.as_bytes(), writer)?,
+			AtomData::UTF16(text) => write_data(DataType::Utf16, text.as_bytes(), writer)?,
 			AtomData::Picture(ref pic) => write_picture(pic, writer)?,
 			AtomData::SignedInteger(int) => write_signed_int(*int, writer)?,
 			AtomData::UnsignedInteger(uint) => write_unsigned_int(*uint, writer)?,
-			AtomData::Bool(b) => write_signed_int(i32::from(*b), writer)?,
+			AtomData::Bool(b) => write_bool(*b, writer)?,
 			AtomData::Unknown { code, ref data } => write_data(*code, data, writer)?,
 		};
 	}
@@ -689,7 +690,7 @@ where
 }
 
 fn write_signed_int(int: i32, writer: &mut AtomWriterCompanion<'_>) -> Result<()> {
-	write_int(21, int.to_be_bytes(), 4, writer)
+	write_int(DataType::BeSignedInteger, int.to_be_bytes(), 4, writer)
 }
 
 fn bytes_to_occupy_uint(uint: u32) -> usize {
@@ -706,11 +707,16 @@ fn bytes_to_occupy_uint(uint: u32) -> usize {
 
 fn write_unsigned_int(uint: u32, writer: &mut AtomWriterCompanion<'_>) -> Result<()> {
 	let bytes_needed = bytes_to_occupy_uint(uint);
-	write_int(22, uint.to_be_bytes(), bytes_needed, writer)
+	write_int(
+		DataType::BeUnsignedInteger,
+		uint.to_be_bytes(),
+		bytes_needed,
+		writer,
+	)
 }
 
 fn write_int(
-	flags: u32,
+	flags: DataType,
 	bytes: [u8; 4],
 	bytes_needed: usize,
 	writer: &mut AtomWriterCompanion<'_>,
@@ -719,15 +725,24 @@ fn write_int(
 	write_data(flags, &bytes[4 - bytes_needed..], writer)
 }
 
+fn write_bool(b: bool, writer: &mut AtomWriterCompanion<'_>) -> Result<()> {
+	write_int(
+		DataType::BeSignedInteger,
+		i32::from(b).to_be_bytes(),
+		1,
+		writer,
+	)
+}
+
 fn write_picture(picture: &Picture, writer: &mut AtomWriterCompanion<'_>) -> Result<()> {
 	match picture.mime_type {
 		// GIF is deprecated
-		Some(MimeType::Gif) => write_data(12, &picture.data, writer),
-		Some(MimeType::Jpeg) => write_data(13, &picture.data, writer),
-		Some(MimeType::Png) => write_data(14, &picture.data, writer),
-		Some(MimeType::Bmp) => write_data(27, &picture.data, writer),
+		Some(MimeType::Gif) => write_data(DataType::Gif, &picture.data, writer),
+		Some(MimeType::Jpeg) => write_data(DataType::Jpeg, &picture.data, writer),
+		Some(MimeType::Png) => write_data(DataType::Png, &picture.data, writer),
+		Some(MimeType::Bmp) => write_data(DataType::Bmp, &picture.data, writer),
 		// We'll assume implicit (0) was the intended type
-		None => write_data(0, &picture.data, writer),
+		None => write_data(DataType::Reserved, &picture.data, writer),
 		_ => Err(FileEncodingError::new(
 			FileType::Mp4,
 			"Attempted to write an unsupported picture format",
@@ -736,8 +751,8 @@ fn write_picture(picture: &Picture, writer: &mut AtomWriterCompanion<'_>) -> Res
 	}
 }
 
-fn write_data(flags: u32, data: &[u8], writer: &mut AtomWriterCompanion<'_>) -> Result<()> {
-	if flags > 16_777_215 {
+fn write_data(flags: DataType, data: &[u8], writer: &mut AtomWriterCompanion<'_>) -> Result<()> {
+	if u32::from(flags) > DataType::MAX {
 		return Err(FileEncodingError::new(
 			FileType::Mp4,
 			"Attempted to write a code that cannot fit in 24 bits",
@@ -755,7 +770,7 @@ fn write_data(flags: u32, data: &[u8], writer: &mut AtomWriterCompanion<'_>) ->
 	// Version
 	writer.write_u8(0)?;
 
-	writer.write_uint::<BigEndian>(u64::from(flags), 3)?;
+	writer.write_u24::<BigEndian>(u32::from(flags))?;
 
 	// Locale
 	writer.write_all(&[0; 4])?;
@@ -766,7 +781,7 @@ fn write_data(flags: u32, data: &[u8], writer: &mut AtomWriterCompanion<'_>) ->
 
 #[cfg(test)]
 mod tests {
-	use crate::mp4::ilst::write::bytes_to_occupy_uint;
+	use super::bytes_to_occupy_uint;
 
 	macro_rules! int_test {
 		(
@@ -787,7 +802,7 @@ mod tests {
 		}
 	}
 
-	#[test]
+	#[test_log::test]
 	fn integer_shrinking_unsigned() {
 		int_test! {
 			func: bytes_to_occupy_uint,
