@@ -3,7 +3,7 @@ use crate::error::{LoftyError, Result};
 use crate::io::{FileLike, Length, Truncate};
 use crate::macros::err;
 use crate::mp4::atom_info::{AtomIdent, AtomInfo, IDENTIFIER_LEN};
-use crate::mp4::read::{meta_is_full, skip_unneeded};
+use crate::mp4::read::{meta_is_full, skip_atom};
 
 use std::cell::{RefCell, RefMut};
 use std::io::{Cursor, Read, Seek, SeekFrom, Write};
@@ -53,7 +53,7 @@ impl ContextualAtom {
 				*reader_len = reader_len.saturating_sub(info.len);
 
 				// We don't care about the atom's contents
-				skip_unneeded(reader, info.extended, info.len)?;
+				skip_atom(reader, info.extended, info.len)?;
 				return Ok(Some(ContextualAtom {
 					info,
 					children: Vec::new(),
@@ -242,19 +242,19 @@ impl AtomWriterCompanion<'_> {
 	}
 }
 
-impl<'a> Seek for AtomWriterCompanion<'a> {
+impl Seek for AtomWriterCompanion<'_> {
 	fn seek(&mut self, pos: SeekFrom) -> std::io::Result<u64> {
 		self.contents.seek(pos)
 	}
 }
 
-impl<'a> Read for AtomWriterCompanion<'a> {
+impl Read for AtomWriterCompanion<'_> {
 	fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
 		self.contents.read(buf)
 	}
 }
 
-impl<'a> Write for AtomWriterCompanion<'a> {
+impl Write for AtomWriterCompanion<'_> {
 	fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
 		self.contents.write(buf)
 	}
