@@ -4,6 +4,7 @@ use crate::config::{ParseOptions, ParsingMode};
 use crate::error::{ErrorKind, LoftyError, Result};
 use crate::macros::{decode_err, err, parse_mode_choice};
 use crate::picture::{MimeType, Picture, PictureInformation, PictureType};
+use crate::tag::Accessor;
 use crate::util::text::{utf16_decode, utf8_decode, utf8_decode_str};
 
 use std::borrow::Cow;
@@ -167,6 +168,39 @@ where
 					},
 				}
 			},
+			// Support the case of TRACKNUMBER being equal to current/total
+			k if k.eq_ignore_ascii_case(b"TRACKNUMBER") => {
+				match utf8_decode_str(value) {
+					Ok(value) => {
+						// try to parse as current/total
+						let mut value_split = value.splitn(2, '/');
+						let track_number: Option<u32> =
+							value_split.next().and_then(|b| b.parse().ok());
+						let track_total: Option<u32> =
+							value_split.next().and_then(|b| b.parse().ok());
+
+						if let Some(n) = track_number {
+							tag.set_track(n);
+						} else {
+							// Probably some other format, like a vinyl track number (A1, B1, etc.).
+							// Just leave it up to the caller to deal with.
+							tag.items
+								.push((String::from("TRACKNUMBER"), value.to_owned()));
+						}
+						if let Some(n) = track_total {
+							tag.set_track_total(n);
+						}
+					},
+					Err(e) => {
+						if parse_mode == ParsingMode::Strict {
+							return Err(e);
+						}
+
+						log::warn!("Non UTF-8 value found, discarding field {key:?}");
+						continue;
+					},
+				}
+			},
 			// The valid range is 0x20..=0x7D not including 0x3D
 			k if k.iter().all(|c| (b' '..=b'}').contains(c) && *c != b'=') => {
 				// SAFETY: We just verified that all of the bytes fall within the subset of ASCII
